@@ -68,6 +68,8 @@ local({
   ## or when explicitly asked for on the command line
   if (interactive() || "--bootstrap-packrat" %in% commandArgs(TRUE)) {
 
+    needsRestore <- "--bootstrap-packrat" %in% commandArgs(TRUE)
+
     message("Packrat is not installed in the local library -- ",
             "attempting to bootstrap an installation...")
 
@@ -117,7 +119,8 @@ local({
       }
 
       # Restore the project, unload the temporary packrat, and load the private packrat
-      packrat::restore(prompt = FALSE, restart = TRUE)
+      if (needsRestore)
+        packrat::restore(prompt = FALSE, restart = TRUE)
 
       ## This code path only reached if we didn't restart earlier
       unloadNamespace("packrat")
@@ -140,7 +143,6 @@ local({
     if (!file.exists(lib)) {
       dir.create(lib, recursive = TRUE)
     }
-    lib <- normalizePath(lib, winslash = "/")
 
     message("> Installing packrat into project private library:")
     message("- ", shQuote(lib))
@@ -150,7 +152,8 @@ local({
       paste0(with, x, with)
     }
 
-    ## The following is performed because a regular install.packages call can fail
+
+    ## Invoke install.packages() in clean R session
     peq <- function(x, y) paste(x, y, sep = " = ")
     installArgs <- c(
       peq("pkgs", surround(packratSrcPath, with = "'")),
@@ -158,17 +161,22 @@ local({
       peq("repos", "NULL"),
       peq("type", surround("source", with = "'"))
     )
-    installCmd <- paste(sep = "",
-                        "utils::install.packages(",
-                        paste(installArgs, collapse = ", "),
-                        ")")
+
+    fmt <- "utils::install.packages(%s)"
+    installCmd <- sprintf(fmt, paste(installArgs, collapse = ", "))
+
+    ## Write script to file (avoid issues with command line quoting
+    ## on R 3.4.3)
+    installFile <- tempfile("packrat-bootstrap", fileext = ".R")
+    writeLines(installCmd, con = installFile)
+    on.exit(unlink(installFile), add = TRUE)
 
     fullCmd <- paste(
       surround(file.path(R.home("bin"), "R"), with = "\""),
       "--vanilla",
       "--slave",
-      "-e",
-      surround(installCmd, with = "\"")
+      "-f",
+      surround(installFile, with = "\"")
     )
     system(fullCmd)
 
@@ -178,10 +186,10 @@ local({
     ## an 'installed from source' version
 
     ## -- InstallAgent -- ##
-    installAgent <- 'InstallAgent: packrat 0.4.8-1'
+    installAgent <- "InstallAgent: packrat 0.4.8-59"
 
     ## -- InstallSource -- ##
-    installSource <- 'InstallSource: source'
+    installSource <- "InstallSource: source"
 
     packratDescPath <- file.path(lib, "packrat", "DESCRIPTION")
     DESCRIPTION <- readLines(packratDescPath)
@@ -193,7 +201,8 @@ local({
     library("packrat", character.only = TRUE, lib.loc = lib)
 
     message("> Restoring library")
-    restore(restart = FALSE)
+    if (needsRestore)
+      packrat::restore(prompt = FALSE, restart = FALSE)
 
     # If the environment allows us to restart, do so with a call to restore
     restart <- getOption("restart")
