@@ -12,3 +12,11 @@ test_that("Rmd documents with parameters are analyzed", {
   expect_true("shiny" %in% deps, "Rmd docs with parameters have a shiny dependency for the customization app")
   expect_true("stringr" %in% deps, "dependencies in parameter expressions are extracted")
 })
+
+test_that("We can discover dependencies with an evaluate hook", {
+
+  path <- "resources/evaluate-deps.Rmd"
+  deps <- fileDependencies.Rmd.evaluate(path)
+  expect_equal(deps, c("abc", "def", "ghi", "jkl"))
+
+})
